@@ -1,6 +1,6 @@
 import time
 import urllib
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, cast
 
 from deprecated.sphinx import versionadded
 from packaging.version import Version
@@ -12,7 +12,7 @@ from limits.util import get_package_data
 
 if TYPE_CHECKING:
     import coredis
-    import coredis.commands.script
+    import coredis.commands
 
 
 class RedisInteractor:
@@ -25,8 +25,10 @@ class RedisInteractor:
     SCRIPT_CLEAR_KEYS = get_package_data(f"{RES_DIR}/clear_keys.lua")
     SCRIPT_INCR_EXPIRE = get_package_data(f"{RES_DIR}/incr_expire.lua")
 
-    lua_moving_window: "coredis.commands.script.Script"  # type: ignore
-    lua_acquire_window: "coredis.commands.script.Script"  # type: ignore
+    lua_moving_window: "coredis.commands.Script[bytes]"
+    lua_acquire_window: "coredis.commands.Script[bytes]"
+    lua_clear_keys: "coredis.commands.Script[bytes]"
+    lua_incr_expire: "coredis.commands.Script[bytes]"
 
     async def _incr(
         self,
@@ -208,7 +210,9 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
                 key, expiry, self.storage, elastic_expiry, amount
             )
         else:
-            return int(await self.lua_incr_expire.execute([key], [expiry, amount]))
+            return cast(
+                int, await self.lua_incr_expire.execute([key], [expiry, amount])
+            )
 
     async def get(self, key: str) -> int:
         """
@@ -260,7 +264,7 @@ class RedisStorage(RedisInteractor, Storage, MovingWindowSupport):
            could be slow on very large data sets.
         """
 
-        return int(await self.lua_clear_keys.execute(["LIMITER*"]))
+        return cast(int, await self.lua_clear_keys.execute(["LIMITER*"]))
 
 
 @versionadded(version="2.1")
@@ -347,6 +351,7 @@ class RedisSentinelStorage(RedisStorage):
         self,
         uri: str,
         service_name: Optional[str] = None,
+        use_replicas: bool = True,
         sentinel_kwargs: Optional[Dict[str, Union[float, str, bool]]] = None,
         **options: Union[float, str, bool],
     ):
@@ -355,6 +360,7 @@ class RedisSentinelStorage(RedisStorage):
          ``async+redis+sentinel://host:port,host:port/service_name``
         :param service_name, optional: sentinel service name
          (if not provided in `uri`)
+        :param use_replicas: Whether to use replicas for read only operations
         :param sentinel_kwargs, optional: kwargs to pass as
          ``sentinel_kwargs`` to :class:`coredis.sentinel.Sentinel`
         :param options: all remaining keyword arguments are passed
@@ -400,6 +406,7 @@ class RedisSentinelStorage(RedisStorage):
         )
         self.storage = self.sentinel.primary_for(self.service_name)
         self.storage_replica = self.sentinel.replica_for(self.service_name)
+        self.use_replicas = use_replicas
         self.initialize_storage(uri)
 
     async def get(self, key: str) -> int:
@@ -407,19 +414,25 @@ class RedisSentinelStorage(RedisStorage):
         :param key: the key to get the counter value for
         """
 
-        return await super()._get(key, self.storage_replica)
+        return await super()._get(
+            key, self.storage_replica if self.use_replicas else self.storage
+        )
 
     async def get_expiry(self, key: str) -> int:
         """
         :param key: the key to get the expiry for
         """
 
-        return await super()._get_expiry(key, self.storage_replica)
+        return await super()._get_expiry(
+            key, self.storage_replica if self.use_replicas else self.storage
+        )
 
     async def check(self) -> bool:
         """
-        Check if storage is healthy by calling :meth:`coredis.StrictRedis.ping`
+        Check if storage is healthy by calling :meth:`coredis.Redis.ping`
         on the replica.
         """
 
-        return await super()._check(self.storage_replica)
+        return await super()._check(
+            self.storage_replica if self.use_replicas else self.storage
+        )
