@@ -1,13 +1,18 @@
 """
 
 """
+import dataclasses
 import re
 import sys
-from typing import Any, Dict, List, Optional, Tuple, Type, Union
+from collections import UserDict
+from types import ModuleType
+from typing import TYPE_CHECKING
 
 import pkg_resources
 from packaging.version import Version
 
+from limits.typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union
+
 from .errors import ConfigurationError
 from .limits import GRANULARITIES, RateLimitItem
 
@@ -28,6 +33,51 @@ EXPR = re.compile(
 )
 
 
+class WindowStats(NamedTuple):
+    """
+    Tuple to describe a rate limited window
+    """
+
+    #: Time as seconds since the Epoch when this window will be reset
+    reset_time: int
+    #: Quantity remaining in this window
+    remaining: int
+
+
+@dataclasses.dataclass
+class Dependency:
+    name: str
+    version_required: Optional[Version]
+    version_found: Optional[Version]
+    module: ModuleType
+
+
+if TYPE_CHECKING:
+    _UserDict = UserDict[str, Dependency]
+else:
+    _UserDict = UserDict
+
+
+class DependencyDict(_UserDict):
+    Missing = Dependency("Missing", None, None, ModuleType("Missing"))
+
+    def __getitem__(self, key: str) -> Dependency:
+        dependency = super().__getitem__(key)
+
+        if dependency == DependencyDict.Missing:
+            raise ConfigurationError(f"{key} prerequisite not available")
+        elif dependency.version_required and (
+            not dependency.version_found
+            or dependency.version_found < dependency.version_required
+        ):
+            raise ConfigurationError(
+                f"The minimum version of {dependency.version_required}"
+                f" of {dependency.name} could not be found"
+            )
+
+        return dependency
+
+
 class LazyDependency:
     """
     Simple utility that provides an :attr:`dependency`
@@ -36,24 +86,26 @@ class LazyDependency:
     """
 
     DEPENDENCIES: Union[Dict[str, Optional[Version]], List[str]] = []
-    FAIL_ON_MISSING_DEPENDENCY: bool = True
     """
     The python modules this class has a dependency on.
     Used to lazily populate the :attr:`dependencies`
     """
 
-    def __init__(self):
-        self._dependencies = {}
+    def __init__(self) -> None:
+        self._dependencies: DependencyDict = DependencyDict()
 
     @property
-    def dependencies(self) -> Dict[str, Any]:
+    def dependencies(self) -> DependencyDict:
         """
         Cached mapping of the modules this storage depends on.
         This is done so that the module is only imported lazily
         when the storage is instantiated.
+
+        :meta private:
         """
-        if not getattr(self, "_dependencies", {}):
-            dependencies = {}
+
+        if not getattr(self, "_dependencies", None):
+            dependencies = DependencyDict()
             mapping: Dict[str, Optional[Version]]
 
             if isinstance(self.DEPENDENCIES, list):
@@ -64,32 +116,28 @@ class LazyDependency:
             for name, minimum_version in mapping.items():
                 dependency, version = get_dependency(name)
 
-                if self.FAIL_ON_MISSING_DEPENDENCY:
-                    if not dependency:
-                        raise ConfigurationError(
-                            f"{name} prerequisite not available"
-                        )  # pragma: no cover
-                    if minimum_version and version < minimum_version:
-                        raise ConfigurationError(
-                            f"The minimum version of {minimum_version} of {name} could not be found"
-                        )
-                if minimum_version and not (version and version >= minimum_version):
-                    dependency = None
-                dependencies[name] = dependency
+                if not dependency:
+                    dependencies[name] = DependencyDict.Missing
+                else:
+                    dependencies[name] = Dependency(
+                        name, minimum_version, version, dependency
+                    )
             self._dependencies = dependencies
+
         return self._dependencies
 
 
-def get_dependency(dep) -> Tuple[Any, Version]:
+def get_dependency(module_path: str) -> Tuple[Optional[ModuleType], Optional[Version]]:
     """
     safe function to import a module at runtime
     """
     try:
-        if dep not in sys.modules:
-            __import__(dep)
-        root = dep.split(".")[0]
+        if module_path not in sys.modules:
+            __import__(module_path)
+        root = module_path.split(".")[0]
         version = getattr(sys.modules[root], "__version__", "0.0.0")
-        return sys.modules[dep], Version(version)
+
+        return sys.modules[module_path], Version(version)
     except ImportError:  # pragma: no cover
         return None, None
 
@@ -138,7 +186,7 @@ def parse(limit_string: str) -> RateLimitItem:
     return list(parse_many(limit_string))[0]
 
 
-def granularity_from_string(granularity_string) -> Type[RateLimitItem]:
+def granularity_from_string(granularity_string: str) -> Type[RateLimitItem]:
     """
 
     :param granularity_string:
